//********************************************************************
//*** ResourcePool/SQL/BindOut.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: BindOut.hh,v 1.6 2005/01/08 20:02:18 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_BINDOUT_HH
#define RESOURCEPOOL_SQL_BINDOUT_HH

#ifndef INCLUDED_GC_PTR_HH
#include "mm/gc_ptr.hh"
#endif

#include <vector>

namespace fatalmind {
namespace SQL {

/**
 * This class defines the common interface for all SQL* classes
 * which do return output colums.
 */
template<class VBC, template <class> class CT>
class BindOut {
	public:
		virtual ~BindOut() {
		}

		/**
		 * Specify where the output data of an select statement
		 * are to be stored.
		 *
		 * You need to specify an place where the columns for an
		 * select statement are to be stored. After each fetch
		 * those variables will be propagated with the actual values
		 * received from the select statement. There is no additional
		 * get() call required, althought the isnull() method is required
		 * to determine if an column contains the NULL value.
		 *
		 * \param pos The column number to be bound. Numbering starts with
		 *        the leftmost column at 0.
		 * \param val the variable which should contain the value of this
		 *        column after the fetch. Any data stored in this variable
		 *        will be overwritten by the fetch(). Exclusive access is
		 *        expected, so there are no synchronization effords taken
		 *        by the select commands.
		 */
		template<typename T>
		void bindout(const unsigned int pos, T& val);

		/**
		 * Used to determine if the column does contain the NULL value
		 * for the row fetched with the most recent fetch operation.
		 *
		 * Actually EVERY column which doesn't have a NOT NULL constraint
		 * must be checked using this method before using the bound variables
		 * content. The effect on the bound variable is UNDEFINED if the
		 * column did contain the NULL value. The variable might be unaffected
		 * but it might also be destroyed in some way.
		 *
		 * \code
		 * SQLSelectRow sel = new SQLSelect("select foo from bar");
		 * std::string foo;
		 * sel.bindout(0, foo);
		 * pool.execute(sel);
		 * if (!sel.isnull(0)) {
		 *    // use foo
		 * } else {
		 *    // column contains the NULL value
		 *    // content of variable foo is UNDEFINED!
		 * }
		 * \endcode
		 *
		 * \param pos The column number to be checked. Same semantic as for the
		 *       bindout() method.
		 * \returns true if the colum contained the NULL value for the most
		 *        recent fetch(), false otherwise.
		 */
		bool isnull(const unsigned int pos) const;

	protected:
		VBC& getOutParam(const unsigned int pos);

		typedef std::vector<gc_ptr<VBC> > _outparamt;
		_outparamt _outparam;

	private:
		void instantiations();
};

} // namespace SQL
} // namespace fatalmind
#endif
