//********************************************************************
//*** ResourcePool/oracle/Resource.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: Resource.cc,v 1.13 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_OCIENVHANDLE_HH
#include "OCIEnvHandle.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

namespace fatalmind {

template<class TM>
oracleResource<TM>::oracleResource(const std::string& service_name
	, const std::string& user
	, const std::string& pass
	, const oracle::OCIEnvHandle& env
	, gc_ptr<_slmdci_t, TM> slmdc
)
: _env(env)
, _err(_env)
, _srv(_env)
, _svc(_env)
, _ses(_env)
, _plain(_env, _err, _srv, _svc, _ses, slmdc)
{
	sword ret;

	/* attach to server */
	if ((ret = OCIServerAttach( _srv.get()
		, _err.get()
		, (OraText *) service_name.c_str()
		, (sb4) service_name.length()
		, OCI_DEFAULT)
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIServerAttach", _err, ret, "");
	}

	/* set server attribute to service context */
	if ((ret = OCIAttrSet((dvoid *) _svc.get()
		, (ub4) OCI_HTYPE_SVCCTX
		, (dvoid *) _srv.get()
		, (ub4) 0
		, (ub4) OCI_ATTR_SERVER
		, _err.get())
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIAttrSet/OCI_HTYPE_SVCCTX/OCI_ATTR_SERVER", _err, ret, "");
	}

	/* set user attribute to session */
	if ((ret = OCIAttrSet ( (dvoid *) _ses.get()
		, OCI_HTYPE_SESSION
		, (dvoid *) user.c_str()
		, (ub4) user.length()
		, (ub4) OCI_ATTR_USERNAME
		, (OCIError *) _err.get())
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIAttrSet/OCI_HTYPE_SESSION/OCI_ATTR_USERNAME", _err, ret, "");
	}

	/* set password attribute to session */
	if ((ret = OCIAttrSet ( (dvoid *) _ses.get()
		, OCI_HTYPE_SESSION
		, (dvoid *) pass.c_str()
		, (ub4) pass.length()
		, (ub4) OCI_ATTR_PASSWORD
		,(OCIError *) _err.get())
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIAttrSet/OCI_HTYPE_SESSION/OCI_ATTR_PASSWORD", _err, ret, "");
	}

	/* Begin a session  */
	if ( (ret = OCISessionBegin (_svc.get()
		, _err.get()
		, _ses.get()
		, OCI_CRED_RDBMS
		, OCI_DEFAULT)
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCISessionBegin", _err, ret, "");
	}

	/* set session attribute to service context */
	if ((ret = OCIAttrSet((dvoid *)_svc.get()
		, (ub4)OCI_HTYPE_SVCCTX
		, (dvoid *)_ses.get()
		, (ub4)0
		, (ub4)OCI_ATTR_SESSION
		, _err.get())
	) != OCI_SUCCESS) {
		OracleExceptionFactory::throwit("OCIAttrSet/OCI_HTYPE_SVCCTX/OCI_ATTR_SESSION", _err, ret, "");
	}
};

template<class TM>
oracleResource<TM>::~oracleResource()
{
	sword ret;
	if ((ret = OCISessionEnd((OCISvcCtx *)_svc.get()
		, (OCIError *)_err.get()
		, (OCISession *)_ses.get()
		, (ub4) OCI_DEFAULT))
	) {
		// TODO: log
	}

	if ((ret = OCIServerDetach((OCIServer *)_srv.get()
		, (OCIError *)_err.get()
		, (ub4)OCI_DEFAULT))
	) {
		// TODO: log
	}
}

template<class TM>
bool oracleResource<TM>::precheck() {
	return true;
}

template<class TM>
bool oracleResource<TM>::postcheck() {
	return true;
}

template<class TM>
void oracleResource<TM>::close() {
	// rely on d-tor
}

template<class TM>
void oracleResource<TM>::fail_close() {
	// rely on d-tor
}

template<class TM>
oracleHandlesWithCache<TM>& oracleResource<TM>::get_plain_resource() {
	return _plain;
}

} // namespace fatalmind
