//********************************************************************
//*** ResourcePool/oracle/SelectListMetaData.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SelectListMetaData.cc,v 1.10 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#include "SelectListMetaData.hh"

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTIONFACTORY_HH
#include "OracleExceptionFactory.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_RESOURCE_HH
#include "Resource.hh"
#endif

namespace fatalmind {
namespace oracle {

ColumnMetaData::ColumnMetaData(const size_t offst) 
: _datatype()
, _datasize()
, _offst(offst)
{
};

ColumnMetaData::ColumnMetaData(const ColumnMetaData& rho) 
: _datatype(rho._datatype)
, _datasize(rho._datasize)
, _offst(rho._offst)
{
}

ub2 ColumnMetaData::getType() const throw()
{
	return _datatype;
}

const ub2* ColumnMetaData::getTypePtr() const throw()
{
	return &_datatype;
}

ub2 ColumnMetaData::getSize() const throw()
{
	return _datasize;
}

const ub2* ColumnMetaData::getSizePtr() const throw()
{
	return &_datasize;
}

size_t ColumnMetaData::getOffset() const throw()
{
	return _offst;
}

bool ColumnMetaData::bufferedDefine() const throw()
{
	return _datatype != SQLT_NUM;
}

bool ColumnMetaData::piecewise() const throw()
{
	return (_datatype == SQLT_CLOB) 
		|| (_datatype == SQLT_BLOB) 
		|| (_datatype == SQLT_FILE) ;
}

std::ostream& operator<<(std::ostream& s, const ColumnMetaData& rho)
{
	return s 
		<< "[ Type: " << rho._datatype << ", Size: " << rho._datasize << "]" 
		<< std::endl;
}

SelectListMetaData::SelectListMetaData(
	  const OCIStatementHandle& sth
	, const oracleHandles& hand
	, const std::string& SQL
):_oraSQL(SQL)
, _bufsz(0)
{
	OCIParam* mypard;
	// TODO retval of OCIParamGet is not really checked
	for (int i = 1
		; OCIParamGet(sth.get()
			, OCI_HTYPE_STMT
			, hand.getError()
			, (void**)&mypard
			, (ub4) i) == OCI_SUCCESS
		; ++i
	) {
		ColumnMetaData cmd(_bufsz);
		sword ret;
		if ((ret = OCIAttrGet((dvoid*) mypard
			, (ub4) OCI_DTYPE_PARAM
			, (dvoid*) cmd.getTypePtr()
			, (ub4 *) 0
			, (ub4) OCI_ATTR_DATA_TYPE
			, (OCIError *) hand.getError() )
		) != OCI_SUCCESS) {
			OracleExceptionFactory::throwit("OCIAttrGet/OCI_DTYPE_PARAM/OCI_ATTR_DATA_TYPE", hand, ret, _oraSQL.getSQL());
		}
		if (cmd.getType() != SQLT_NUM) {
			if ((ret = OCIAttrGet((dvoid*) mypard
				, (ub4) OCI_DTYPE_PARAM
				, (dvoid*) cmd.getSizePtr()
				, (ub4 *) 0
				, (ub4) OCI_ATTR_DATA_SIZE
				, (OCIError *) hand.getError() )
			) != OCI_SUCCESS) {
				OracleExceptionFactory::throwit("OCIAttrGet/OCI_DTYPE_PARAM/OCI_ATTR_DATA_SIZE", hand, ret, _oraSQL.getSQL());
			}
			_bufsz += cmd.getSize() + sizeof(DefineMetaData);
		}
		_selectlist.push_back(cmd);
	}
}

size_t SelectListMetaData::getBufferSz() const throw()
{
	return _bufsz;
}

int SelectListMetaData::getNumSelectListItems() const
{
	return _selectlist.size();
}

const ColumnMetaData& SelectListMetaData::getColumnMetaData(const int pos) const
{
	return _selectlist[pos];
}
const oracleStatement& SelectListMetaData::getOraSQL() const
{
	return _oraSQL;
}

std::ostream& operator<<(std::ostream& s, const SelectListMetaData& rho) 
{
	s << "SelectListMetaData for '" << rho._oraSQL.getSQL() << "'" << std::endl;
	SelectListMetaData::selectlist_t::const_iterator end = rho._selectlist.end();
	for (SelectListMetaData::selectlist_t::const_iterator i = rho._selectlist.begin()
		; i != end
		; ++i
	) {
		s << "   " << *i;
	}
	return s;
}

SelectListBuffer::SelectListBuffer(const gc_ptr<SelectListMetaData> slmd, oracleBindOut& bindout)
: _slmd(slmd)
{
	_buf = (char*)operator new(_slmd->getBufferSz());
	int no = slmd->getNumSelectListItems();
	for (int i = 0; i < no; ++i) {
		const ColumnMetaData& cmd = slmd->getColumnMetaData(i);
		if (cmd.bufferedDefine()) {
			// placement new
			new(_buf + cmd.getOffset())DefineMetaData(cmd.getSize(), bindout.getOutParam(i));
		}
	}
}

SelectListBuffer::~SelectListBuffer()
{
	operator delete(_buf, std::nothrow_t());
}

int SelectListBuffer::getNumSelectListItems() const
{
	return _slmd->getNumSelectListItems();
}

DefineMetaData* SelectListBuffer::getDefineMetaData(const int pos) const
{
	const ColumnMetaData& cmd(_slmd->getColumnMetaData(pos));
	if (cmd.bufferedDefine()) {
		return (DefineMetaData*)(_buf + cmd.getOffset());
	} else {
		return 0;
	}
}

ub4* SelectListBuffer::getSizePtr(const int pos) const 
{
	return getDefineMetaData(pos)->getSizePtr();
}

void* SelectListBuffer::getDataPtr(const int pos) const
{
	return getDefineMetaData(pos)->getDataPtr();
}

bool SelectListBuffer::bufferedDefine(const int pos) const
{
	return _slmd->getColumnMetaData(pos).bufferedDefine();
}

} // namespace oracle
} // namespace fatalmind
